@@ -1,26 +1,27 @@
 //! Traits to represent a Pact
 
-use std::borrow::Borrow;
-use std::collections::BTreeMap;
+use std::collections::{BTreeMap, HashMap};
 use std::fmt::Debug;
 use std::fs;
 use std::fs::File;
 use std::io::{Read, Seek, SeekFrom, Write};
+use std::ops::Deref;
 use std::path::Path;
 use std::sync::{Arc, Mutex};
 
 use anyhow::{anyhow, Context};
 use lazy_static::lazy_static;
-use log::{debug, error, warn};
+use log::{debug, error, warn, trace};
 use maplit::btreemap;
 use serde_json::{json, Value};
 
 use crate::{Consumer, PactSpecification, Provider};
-#[cfg(not(target_family = "wasm"))] use crate::http_utils;
 #[cfg(not(target_family = "wasm"))] use crate::file_utils::{with_read_lock_for_open_file, with_write_lock};
+#[cfg(not(target_family = "wasm"))] use crate::http_utils;
 #[cfg(not(target_family = "wasm"))] use crate::http_utils::HttpAuth;
 use crate::interaction::Interaction;
 use crate::message_pact::MessagePact;
+use crate::plugins::PluginData;
 use crate::sync_pact::RequestResponsePact;
 use crate::v4;
 use crate::v4::pact::V4Pact;
@@ -30,30 +31,64 @@ use crate::verify_json::{json_type_of, PactFileVerificationResult, ResultLevel};
 pub trait Pact: Debug + ReadWritePact {
   /// Consumer side of the pact
   fn consumer(&self) -> Consumer;
+
   /// Provider side of the pact
   fn provider(&self) -> Provider;
+
   /// Interactions in the Pact
-  fn interactions(&self) -> Vec<&dyn Interaction>;
+  fn interactions(&self) -> Vec<Box<dyn Interaction + Send + Sync>>;
+
   /// Pact metadata
   fn metadata(&self) -> BTreeMap<String, BTreeMap<String, String>>;
+
   /// Converts this pact to a `Value` struct.
   fn to_json(&self, pact_spec: PactSpecification) -> anyhow::Result<Value>;
+
   /// Attempt to downcast to a concrete Pact
   fn as_request_response_pact(&self) -> anyhow::Result<RequestResponsePact>;
+
   /// Attempt to downcast to a concrete Message Pact
   fn as_message_pact(&self) -> anyhow::Result<MessagePact>;
+
   /// Attempt to downcast to a concrete V4 Pact
   fn as_v4_pact(&self) -> anyhow::Result<V4Pact>;
+
   /// Specification version of this Pact
   fn specification_version(&self) -> PactSpecification;
+
   /// Clones this Pact and wraps it in a Box
-  fn boxed(&self) -> Box<dyn Pact + Send>;
+  fn boxed(&self) -> Box<dyn Pact + Send + Sync>;
+
   /// Clones this Pact and wraps it in an Arc
-  fn arced(&self) -> Arc<dyn Pact + Send>;
+  fn arced(&self) -> Arc<dyn Pact + Send + Sync>;
+
   /// Clones this Pact and wraps it in an Arc and Mutex
   fn thread_safe(&self) -> Arc<Mutex<dyn Pact + Send + Sync>>;
+
   /// Adds an interactions in the Pact
   fn add_interaction(&mut self, interaction: &dyn Interaction) -> anyhow::Result<()>;
+
+  /// If this Pact needs any plugins loaded
+  fn requires_plugins(&self) -> bool;
+
+  /// Plugins required for this Pact. These will be taken from the 'plugins' key in the pact
+  /// metadata.
+  fn plugin_data(&self) -> Vec<PluginData>;
+
+  /// If this is a V4 Pact
+  fn is_v4(&self) -> bool;
+
+  /// Add the plugin and plugin data to this Pact. If an entry already exists for the plugin,
+  /// the plugin data will be merged
+  fn add_plugin(
+    &mut self,
+    name: &str,
+    version: &str,
+    plugin_data: Option<HashMap<String, Value>>
+  ) -> anyhow::Result<()>;
+
+  /// Adds some version info to the Pact-Rust metadata section
+  fn add_md_version(&mut self, key: &str, version: &str);
 }
 
 impl Default for Box<dyn Pact> {
@@ -96,14 +131,14 @@ impl PartialEq for Box<dyn Pact> {
 
 /// Reads the pact file and parses the resulting JSON into a `Pact` struct
 #[cfg(not(target_family = "wasm"))]
-pub fn read_pact(file: &Path) -> anyhow::Result<Box<dyn Pact>> {
+pub fn read_pact(file: &Path) -> anyhow::Result<Box<dyn Pact + Send + Sync>> {
   let mut f = File::open(file)?;
   read_pact_from_file(&mut f, file)
 }
 
 /// Reads the pact from the file and parses the resulting JSON into a `Pact` struct
 #[cfg(not(target_family = "wasm"))]
-pub fn read_pact_from_file(file: &mut File, path: &Path) -> anyhow::Result<Box<dyn Pact>> {
+pub fn read_pact_from_file(file: &mut File, path: &Path) -> anyhow::Result<Box<dyn Pact + Send + Sync>> {
   let buf = with_read_lock_for_open_file(path, file, 3, &mut |f| {
     let mut buf = String::new();
     f.read_to_string(&mut buf)?;
@@ -122,23 +157,28 @@ pub fn read_pact_from_file(file: &mut File, path: &Path) -> anyhow::Result<Box<d
 
 /// Reads the pact file from a URL and parses the resulting JSON into a `Pact` struct
 #[cfg(not(target_family = "wasm"))]
-pub fn load_pact_from_url(url: &str, auth: &Option<HttpAuth>) -> anyhow::Result<Box<dyn Pact>> {
+pub fn load_pact_from_url(url: &str, auth: &Option<HttpAuth>) -> anyhow::Result<Box<dyn Pact + Send + Sync>> {
   let (url, pact_json) = http_utils::fetch_json_from_url(&url.to_string(), auth)?;
   load_pact_from_json(&url, &pact_json)
 }
 
 /// Loads a Pact model from a JSON Value
-pub fn load_pact_from_json(source: &str, json: &Value) -> anyhow::Result<Box<dyn Pact>> {
+pub fn load_pact_from_json(source: &str, json: &Value) -> anyhow::Result<Box<dyn Pact + Send + Sync>> {
   match json {
-    Value::Object(map) => if map.contains_key("messages") {
-      let pact = MessagePact::from_json(source, json)?;
-      Ok(Box::new(pact))
-    } else {
+    Value::Object(map) => {
       let metadata = parse_meta_data(json);
       let spec_version = determine_spec_version(source, &metadata);
+      trace!("load_pact_from_json: found spec version {} in metadata", spec_version);
       match spec_version {
         PactSpecification::V4 => v4::pact::from_json(&source, json),
-        _ => Ok(Box::new(RequestResponsePact::from_json(source, json)?))
+        _ => if map.contains_key("messages") {
+          trace!("load_pact_from_json: JSON has a messages attribute, will load as a message pact");
+          let pact = MessagePact::from_json(source, json)?;
+          Ok(Box::new(pact))
+        } else {
+          trace!("load_pact_from_json: loading JSON as a request/response pact");
+          Ok(Box::new(RequestResponsePact::from_json(source, json)?))
+        }
       }
     },
     _ => Err(anyhow!("Failed to parse Pact JSON from source '{}' - it is not a valid pact file", source))
@@ -149,13 +189,13 @@ pub fn load_pact_from_json(source: &str, json: &Value) -> anyhow::Result<Box<dyn
 pub trait ReadWritePact {
   /// Reads the pact file and parses the resulting JSON into a `Pact` struct
   #[cfg(not(target_family = "wasm"))]
-  fn read_pact(path: &Path) -> anyhow::Result<Self> where Self: std::marker::Sized;
+  fn read_pact(path: &Path) -> anyhow::Result<Self> where Self: std::marker::Sized + Send + Sync;
 
   /// Merges this pact with the other pact, and returns a new Pact with the interactions sorted.
   /// Returns an error if there is a merge conflict, which will occur if the other pact is a different
   /// type, or if a V3 Pact then if any interaction has the
   /// same description and provider state and the requests and responses are different.
-  fn merge(&self, other: &dyn Pact) -> anyhow::Result<Box<dyn Pact>>;
+  fn merge(&self, other: &dyn Pact) -> anyhow::Result<Box<dyn Pact + Send + Sync>>;
 
   /// Determines the default file name for the pact. This is based on the consumer and
   /// provider names.
@@ -188,7 +228,7 @@ pub fn write_pact(
             existing_pact.specification_version());
     }
 
-    let merged_pact = pact.merge(existing_pact.borrow())?;
+    let merged_pact = pact.merge(existing_pact.deref())?;
     let pact_json = serde_json::to_string_pretty(&merged_pact.to_json(pact_spec)?)?;
 
     with_write_lock(path, &mut f, 3, &mut |f| {
@@ -390,6 +430,7 @@ mod tests {
 
   use expectest::prelude::*;
   use maplit::{btreemap, hashmap};
+  use pretty_assertions::assert_eq;
   use serde_json::{json, Value};
 
   use crate::{Consumer, PactSpecification, Provider};
@@ -553,7 +594,7 @@ mod tests {
     expect!(interaction.response).to(be_equal_to(Response {
       status: 200,
       headers: Some(hashmap!{ "Content-Type".to_string() => vec!["text/html".to_string()] }),
-      body: OptionalBody::Present("\"That is some good Mallory.\"".into(), Some("text/html".into())),
+      body: OptionalBody::Present("\"That is some good Mallory.\"".into(), Some("text/html".into()), None),
       .. Response::default()
     }));
     expect!(pact.specification_version).to(be_equal_to(PactSpecification::V3));
@@ -698,13 +739,13 @@ mod tests {
       path: "/".to_string(),
       query: Some(hashmap!{ "q".to_string() => vec!["p".to_string(), "p2".to_string()], "r".to_string() => vec!["s".to_string()] }),
       headers: Some(hashmap!{ "testreqheader".to_string() => vec!["testreqheadervalue".to_string()] }),
-      body: OptionalBody::Present("{\"test\":true}".into(), None),
+      body: OptionalBody::Present("{\"test\":true}".into(), None, None),
       .. Request::default()
     }));
     expect!(interaction.response).to(be_equal_to(Response {
       status: 200,
       headers: Some(hashmap!{ "testreqheader".to_string() => vec!["testreqheaderval".to_string()] }),
-      body: OptionalBody::Present("{\"responsetest\":true}".into(), None),
+      body: OptionalBody::Present("{\"responsetest\":true}".into(), None, None),
       .. Response::default()
     }));
   }
@@ -764,7 +805,7 @@ mod tests {
       query: Some(hashmap!{ "datetime".to_string() => vec!["2011-12-03T10:15:30+01:00".to_string()],
             "description".to_string() => vec!["hello world!".to_string()] }),
       headers: Some(hashmap!{ "testreqheader".to_string() => vec!["testreqheadervalue".to_string()] }),
-      body: OptionalBody::Present("{\"test\":true}".into(), None),
+      body: OptionalBody::Present("{\"test\":true}".into(), None, None),
       .. Request::default()
     }));
   }
@@ -841,7 +882,7 @@ mod tests {
     fs::remove_dir_all(dir.parent().unwrap()).unwrap_or(());
 
     expect!(result).to(be_ok());
-    expect!(pact_file).to(be_equal_to(format!(r#"{{
+    assert_eq!(pact_file, format!(r#"{{
   "consumer": {{
     "name": "write_pact_test_consumer"
   }},
@@ -860,7 +901,7 @@ mod tests {
   ],
   "metadata": {{
     "pactRust": {{
-      "version": "{}"
+      "models": "{}"
     }},
     "pactSpecification": {{
       "version": "2.0.0"
@@ -869,7 +910,7 @@ mod tests {
   "provider": {{
     "name": "write_pact_test_provider"
   }}
-}}"#, PACT_RUST_VERSION.unwrap())));
+}}"#, PACT_RUST_VERSION.unwrap()));
   }
 
   #[test]
@@ -941,7 +982,7 @@ mod tests {
   ],
   "metadata": {{
     "pactRust": {{
-      "version": "{}"
+      "models": "{}"
     }},
     "pactSpecification": {{
       "version": "2.0.0"
@@ -1012,7 +1053,7 @@ mod tests {
   ],
   "metadata": {{
     "pactRust": {{
-      "version": "{}"
+      "models": "{}"
     }},
     "pactSpecification": {{
       "version": "2.0.0"
@@ -1101,7 +1142,7 @@ mod tests {
   ],
   "metadata": {{
     "pactRust": {{
-      "version": "{}"
+      "models": "{}"
     }},
     "pactSpecification": {{
       "version": "3.0.0"
@@ -1140,7 +1181,7 @@ mod tests {
           .. Default::default()
         })
       ],
-      metadata: btreemap! {},
+      .. V4Pact::default()
     };
     let mut dir = env::temp_dir();
     let x = rand::random::<u16>();
@@ -1199,7 +1240,7 @@ mod tests {
   ],
   "metadata": {{
     "pactRust": {{
-      "version": "{}"
+      "models": "{}"
     }},
     "pactSpecification": {{
       "version": "4.0"
@@ -1367,7 +1408,7 @@ mod tests {
   ],
   "metadata": {{
     "pactRust": {{
-      "version": "{}"
+      "models": "{}"
     }},
     "pactSpecification": {{
       "version": "2.0.0"
@@ -1463,7 +1504,7 @@ mod tests {
       ],
       "metadata": {
         "pactRust": {
-          "version": PACT_RUST_VERSION
+          "models": PACT_RUST_VERSION
         },
         "pactSpecification": {
           "version": "3.0.0"
@@ -1539,7 +1580,7 @@ mod tests {
   ],
   "metadata": {{
     "pactRust": {{
-      "version": "{}"
+      "models": "{}"
     }},
     "pactSpecification": {{
       "version": "3.0.0"
@@ -1623,7 +1664,7 @@ mod tests {
   ],
   "metadata": {{
     "pactRust": {{
-      "version": "{}"
+      "models": "{}"
     }},
     "pactSpecification": {{
       "version": "3.0.0"
@@ -1642,7 +1683,7 @@ mod tests {
         RequestResponseInteraction {
           description: "Test Interaction with matcher".to_string(),
           request: Request {
-            body: OptionalBody::Present(json!({ "related": [1, 2, 3] }).to_string().into(), Some(JSON.clone())),
+            body: OptionalBody::Present(json!({ "related": [1, 2, 3] }).to_string().into(), Some(JSON.clone()), None),
             matching_rules: matchingrules!{
             "body" => {
               "$.related" => [ MatchingRule::MinMaxType(0, 5) ]
@@ -1659,7 +1700,7 @@ mod tests {
         RequestResponseInteraction {
           description: "Test Interaction with matcher".to_string(),
           request: Request {
-            body: OptionalBody::Present(json!({ "related": [1, 2, 3] }).to_string().into(), Some(JSON.clone())),
+            body: OptionalBody::Present(json!({ "related": [1, 2, 3] }).to_string().into(), Some(JSON.clone()), None),
             matching_rules: matchingrules!{
             "body" => {
               "$.related" => [ MatchingRule::MinMaxType(1, 10) ]
