@@ -54,7 +54,10 @@ pub struct AsynchronousMessage {
   pub plugin_config: HashMap<String, HashMap<String, Value>>,
 
   /// Text markup to use to render the interaction in a UI
-  pub interaction_markup: InteractionMarkup
+  pub interaction_markup: InteractionMarkup,
+
+  /// Transport mechanism used with this message
+  pub transport: Option<String>
 }
 
 impl AsynchronousMessage {
@@ -116,6 +119,13 @@ impl AsynchronousMessage {
       let interaction_markup = json.get("interactionMarkup")
         .map(|markup| InteractionMarkup::from_json(markup)).unwrap_or_default();
 
+      let transport = json.get("transport").map(|value| {
+        match value {
+          Value::String(s) => s.clone(),
+          _ => value.to_string()
+        }
+      });
+
       Ok(AsynchronousMessage {
         id,
         key,
@@ -131,7 +141,8 @@ impl AsynchronousMessage {
         pending: json.get("pending")
           .map(|value| value.as_bool().unwrap_or_default()).unwrap_or_default(),
         plugin_config,
-        interaction_markup
+        interaction_markup,
+        transport
       })
     } else {
       Err(anyhow!("Expected a JSON object for the interaction, got '{}'", json))
@@ -198,6 +209,11 @@ impl V4Interaction for AsynchronousMessage {
       map.insert("interactionMarkup".to_string(), self.interaction_markup.to_json());
     }
 
+    if let Some(transport) = &self.transport {
+      let map = json.as_object_mut().unwrap();
+      map.insert("transport".to_string(), Value::String(transport.clone()));
+    }
+
     json
   }
 
@@ -240,6 +256,14 @@ impl V4Interaction for AsynchronousMessage {
   fn interaction_markup_mut(&mut self) -> &mut InteractionMarkup {
     &mut self.interaction_markup
   }
+
+  fn transport(&self) -> Option<String> {
+    self.transport.clone()
+  }
+
+  fn set_transport(&mut self, transport: Option<String>) {
+    self.transport = transport.clone();
+  }
 }
 
 impl Interaction for AsynchronousMessage {
@@ -391,7 +415,8 @@ impl Default for AsynchronousMessage {
       comments: Default::default(),
       pending: false,
       plugin_config: Default::default(),
-      interaction_markup: Default::default()
+      interaction_markup: Default::default(),
+      transport: None
     }
   }
 }
