@@ -51,7 +51,10 @@ pub struct SynchronousMessage {
   pub plugin_config: HashMap<String, HashMap<String, Value>>,
 
   /// Text markup to use to render the interaction in a UI
-  pub interaction_markup: InteractionMarkup
+  pub interaction_markup: InteractionMarkup,
+
+  /// Transport mechanism used with this message
+  pub transport: Option<String>
 }
 
 impl SynchronousMessage {
@@ -110,6 +113,13 @@ impl SynchronousMessage {
       let interaction_markup = json.get("interactionMarkup")
         .map(|markup| InteractionMarkup::from_json(markup)).unwrap_or_default();
 
+      let transport = json.get("transport").map(|value| {
+        match value {
+          Value::String(s) => s.clone(),
+          _ => value.to_string()
+        }
+      });
+
       if responses.iter().any(|res| res.is_err()) {
         let errors = responses.iter()
           .filter(|res| res.is_err())
@@ -128,7 +138,8 @@ impl SynchronousMessage {
           pending: json.get("pending")
             .map(|value| value.as_bool().unwrap_or_default()).unwrap_or_default(),
           plugin_config,
-          interaction_markup
+          interaction_markup,
+          transport
         })
       }
     } else {
@@ -176,6 +187,11 @@ impl V4Interaction for SynchronousMessage {
       map.insert("interactionMarkup".to_string(), self.interaction_markup.to_json());
     }
 
+    if let Some(transport) = &self.transport {
+      let map = json.as_object_mut().unwrap();
+      map.insert("transport".to_string(), Value::String(transport.clone()));
+    }
+
     json
   }
 
@@ -218,6 +234,14 @@ impl V4Interaction for SynchronousMessage {
   fn interaction_markup_mut(&mut self) -> &mut InteractionMarkup {
     &mut self.interaction_markup
   }
+
+  fn transport(&self) -> Option<String> {
+    self.transport.clone()
+  }
+
+  fn set_transport(&mut self, transport: Option<String>) {
+    self.transport = transport.clone();
+  }
 }
 
 impl Interaction for SynchronousMessage {
@@ -350,7 +374,8 @@ impl Default for SynchronousMessage {
       response: Default::default(),
       pending: false,
       plugin_config: Default::default(),
-      interaction_markup: Default::default()
+      interaction_markup: Default::default(),
+      transport: None
     }
   }
 }
